require "import"
import "android.widget.*"
import "android.view.*"
import "android.text.InputType"
import "android.speech.tts.TextToSpeech"
import "java.util.Locale"

local activity = activity

-- Create UI elements
local edt1, edt2, txtResult, btnAdd, btnSubtract, btnMultiply, btnDivide, btnPercentage, btnClear, tts

function onCreate(savedInstanceState)
    activity.setTitle("new Calculator")
    -- Initialize TextToSpeech engine
    tts = TextToSpeech(activity, nil)
    tts.setLanguage(Locale.US)

    local layout = LinearLayout(activity)
    layout.setOrientation(1)
    
    edt1 = EditText(activity)
    edt1.setHint("Enter first number")
    edt1.setInputType(InputType.TYPE_CLASS_NUMBER | InputType.TYPE_NUMBER_FLAG_DECIMAL)
    layout.addView(edt1)

    edt2 = EditText(activity)
    edt2.setHint("Enter second number")
    edt2.setInputType(InputType.TYPE_CLASS_NUMBER | InputType.TYPE_NUMBER_FLAG_DECIMAL)
    layout.addView(edt2)
    
    txtResult = TextView(activity)
    txtResult.setTextSize(18)
    layout.addView(txtResult)
    
    btnAdd = Button(activity)
    btnAdd.setText("Add")
    btnAdd.setOnClickListener{onClick = function(view) calculate('add') end}
    layout.addView(btnAdd)
    
    btnSubtract = Button(activity)
    btnSubtract.setText("Subtract")
    btnSubtract.setOnClickListener{onClick = function(view) calculate('subtract') end}
    layout.addView(btnSubtract)
    
    btnMultiply = Button(activity)
    btnMultiply.setText("Multiply")
    btnMultiply.setOnClickListener{onClick = function(view) calculate('multiply') end}
    layout.addView(btnMultiply)

    btnDivide = Button(activity)
    btnDivide.setText("Divide")
    btnDivide.setOnClickListener{onClick = function(view) calculate('divide') end}
    layout.addView(btnDivide)

    btnPercentage = Button(activity)
    btnPercentage.setText("Calculate Percentage")
    btnPercentage.setOnClickListener{onClick = function(view) calculate('percentage') end}
    layout.addView(btnPercentage)
    
    -- Add clear button
    btnClear = Button(activity)
    btnClear.setText("Clear")
    btnClear.setOnClickListener{onClick = function(view) clearFields() end}
    layout.addView(btnClear)

    activity.setContentView(layout)
end

function calculate(operation)
    local num1 = tonumber(edt1.getText().toString())
    local num2 = tonumber(edt2.getText().toString())
    local result, speechText

    if operation == 'add' then
        result = num1 + num2
        speechText = num1 .. " plus " .. num2 .. " equals " .. result
    elseif operation == 'subtract' then
        result = num1 - num2
        speechText = num1 .. " minus " .. num2 .. " equals " .. result
    elseif operation == 'multiply' then
        result = num1 * num2
        speechText = num1 .. " times " .. num2 .. " equals " .. result
    elseif operation == 'divide' then
        if num2 ~= 0 then
            result = num1 / num2
            speechText = num1 .. " divided by " .. num2 .. " equals " .. result
        else
            result = "Error: Division by zero"
            speechText = "Division by zero is not allowed."
        end
    elseif operation == 'percentage' then
        result = (num1 / 100) * num2
        speechText = num2 .. " percent of " .. num1 .. " equals " .. result
    end
    
    txtResult.setText(tostring(result))
    if tts and speechText then
        tts.speak(speechText, TextToSpeech.QUEUE_FLUSH, nil, nil)
    end
end

function clearFields()
    edt1.setText("")
    edt2.setText("")
    txtResult.setText("")
  
    if tts then
        tts.speak("cleared", TextToSpeech.QUEUE_FLUSH, nil, nil)
    end
end

function onDestroy()
    if tts ~= nil then
        tts.stop()
        tts.shutdown()
    end
end
